"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.usePrerenderedData = void 0;

var _react = _interopRequireWildcard(require("react"));

var _reactNative = require("react-native");

var _config = require("../config");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const Prerender = _ref => {
  let {
    id,
    onLayout,
    children
  } = _ref;
  const onLayoutView = (0, _react.useCallback)(e => {
    onLayout({
      id,
      height: e.nativeEvent.layout.height
    });
  }, [id, onLayout]);
  return /*#__PURE__*/_react.default.createElement(_reactNative.View, {
    onLayout: onLayoutView
  }, children);
};

const usePrerenderedData = _ref2 => {
  let {
    data,
    keyExtractor,
    renderItem,
    scrollRef,
    onUpdateData,
    getItemLayout
  } = _ref2;
  const [finalData, setFinalData] = (0, _react.useState)([]);
  const [newData, setNewData] = (0, _react.useState)([]);
  const heightsRef = (0, _react.useRef)({});
  const getHeight = (0, _react.useCallback)(function (list) {
    let heights = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : heightsRef.current;
    return list.reduce((p, c) => p + heights[keyExtractor(c)], 0);
  }, [keyExtractor]);
  const shift = (0, _react.useCallback)((newData, oldHeights) => {
    const removedData = [];

    for (const d of finalData) {
      if (!heightsRef.current[keyExtractor(d)]) {
        removedData.push(d);
      } else {
        break;
      }
    }

    const shiftValue = {
      height: -getHeight(removedData, oldHeights),
      offset: 0
    };
    const index = data.findIndex(d => d === newData[0]);

    if (index >= 0 && index <= _config.MIN_INDEX) {
      shiftValue.height += getHeight(newData);
      shiftValue.offset = getHeight(data.slice(0, index));
    }

    if (shiftValue.height !== 0) {
      var _scrollRef$current;

      (_scrollRef$current = scrollRef.current) === null || _scrollRef$current === void 0 ? void 0 : _scrollRef$current.shift(shiftValue);
      onUpdateData === null || onUpdateData === void 0 ? void 0 : onUpdateData({
        heights: heightsRef.current,
        ...shiftValue
      });
    }

    setFinalData(data);
  }, [data, finalData, getHeight, keyExtractor, onUpdateData, scrollRef]);
  (0, _react.useEffect)(() => {
    if (data === finalData) {
      return;
    }

    if (getItemLayout) {
      const newD = data.filter(d => !heightsRef.current[keyExtractor(d)]);
      const oldHeights = heightsRef.current;
      heightsRef.current = data.reduce((p, c, i) => {
        p[keyExtractor(c)] = getItemLayout(data, i).length;
        return p;
      }, {});
      shift(newD, oldHeights);
      return;
    }

    setNewData(data.filter(d => !heightsRef.current[keyExtractor(d)]));
  }, [data, finalData, getHeight, getItemLayout, keyExtractor, onUpdateData, scrollRef, shift]);
  (0, _react.useEffect)(() => {
    if (getItemLayout || !newData.length) {
      return;
    }
  }, [newData, getItemLayout]);
  const onLayout = (0, _react.useCallback)(_ref3 => {
    let {
      id,
      height
    } = _ref3;
    heightsRef.current[id] = height; // check if there are missing elements

    const missing = data.some(d => heightsRef.current[keyExtractor(d)] === undefined);

    if (missing) {
      return;
    }

    const oldHeights = heightsRef.current; // clean current heights (=> remove old heights)

    heightsRef.current = data.reduce((p, c) => {
      const id = keyExtractor(c);
      p[id] = oldHeights[id];
      return p;
    }, {});
    shift(newData, oldHeights);
    setNewData([]);
  }, [data, keyExtractor, newData, shift]);
  return {
    finalData,
    prerender: newData.length ? /*#__PURE__*/_react.default.createElement(_reactNative.View, null, newData.map(d => /*#__PURE__*/_react.default.createElement(Prerender, {
      key: keyExtractor(d),
      id: keyExtractor(d),
      onLayout: onLayout
    }, renderItem({
      item: d,
      prerendering: true
    })))) : undefined,
    getItemLayoutCustom: (0, _react.useCallback)((data, index) => {
      const d = data[index];
      const id = keyExtractor(d);
      return {
        index,
        length: heightsRef.current[id],
        offset: getHeight(data.slice(0, index))
      };
    }, [getHeight, keyExtractor])
  };
};

exports.usePrerenderedData = usePrerenderedData;
//# sourceMappingURL=usePrerenderedData.js.map