import React, { useCallback, useEffect, useRef, useState } from 'react';
import { View } from 'react-native';
import { MIN_INDEX } from '../config';

const Prerender = _ref => {
  let {
    id,
    onLayout,
    children
  } = _ref;
  const onLayoutView = useCallback(e => {
    onLayout({
      id,
      height: e.nativeEvent.layout.height
    });
  }, [id, onLayout]);
  return /*#__PURE__*/React.createElement(View, {
    onLayout: onLayoutView
  }, children);
};

export const usePrerenderedData = _ref2 => {
  let {
    data,
    keyExtractor,
    renderItem,
    scrollRef,
    onUpdateData,
    getItemLayout
  } = _ref2;
  const [finalData, setFinalData] = useState([]);
  const [newData, setNewData] = useState([]);
  const heightsRef = useRef({});
  const getHeight = useCallback(function (list) {
    let heights = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : heightsRef.current;
    return list.reduce((p, c) => p + heights[keyExtractor(c)], 0);
  }, [keyExtractor]);
  const shift = useCallback((newData, oldHeights) => {
    const removedData = [];

    for (const d of finalData) {
      if (!heightsRef.current[keyExtractor(d)]) {
        removedData.push(d);
      } else {
        break;
      }
    }

    const shiftValue = {
      height: -getHeight(removedData, oldHeights),
      offset: 0
    };
    const index = data.findIndex(d => d === newData[0]);

    if (index >= 0 && index <= MIN_INDEX) {
      shiftValue.height += getHeight(newData);
      shiftValue.offset = getHeight(data.slice(0, index));
    }

    if (shiftValue.height !== 0) {
      var _scrollRef$current;

      (_scrollRef$current = scrollRef.current) === null || _scrollRef$current === void 0 ? void 0 : _scrollRef$current.shift(shiftValue);
      onUpdateData === null || onUpdateData === void 0 ? void 0 : onUpdateData({
        heights: heightsRef.current,
        ...shiftValue
      });
    }

    setFinalData(data);
  }, [data, finalData, getHeight, keyExtractor, onUpdateData, scrollRef]);
  useEffect(() => {
    if (data === finalData) {
      return;
    }

    if (getItemLayout) {
      const newD = data.filter(d => !heightsRef.current[keyExtractor(d)]);
      const oldHeights = heightsRef.current;
      heightsRef.current = data.reduce((p, c, i) => {
        p[keyExtractor(c)] = getItemLayout(data, i).length;
        return p;
      }, {});
      shift(newD, oldHeights);
      return;
    }

    setNewData(data.filter(d => !heightsRef.current[keyExtractor(d)]));
  }, [data, finalData, getHeight, getItemLayout, keyExtractor, onUpdateData, scrollRef, shift]);
  useEffect(() => {
    if (getItemLayout || !newData.length) {
      return;
    }
  }, [newData, getItemLayout]);
  const onLayout = useCallback(_ref3 => {
    let {
      id,
      height
    } = _ref3;
    heightsRef.current[id] = height; // check if there are missing elements

    const missing = data.some(d => heightsRef.current[keyExtractor(d)] === undefined);

    if (missing) {
      return;
    }

    const oldHeights = heightsRef.current; // clean current heights (=> remove old heights)

    heightsRef.current = data.reduce((p, c) => {
      const id = keyExtractor(c);
      p[id] = oldHeights[id];
      return p;
    }, {});
    shift(newData, oldHeights);
    setNewData([]);
  }, [data, keyExtractor, newData, shift]);
  return {
    finalData,
    prerender: newData.length ? /*#__PURE__*/React.createElement(View, null, newData.map(d => /*#__PURE__*/React.createElement(Prerender, {
      key: keyExtractor(d),
      id: keyExtractor(d),
      onLayout: onLayout
    }, renderItem({
      item: d,
      prerendering: true
    })))) : undefined,
    getItemLayoutCustom: useCallback((data, index) => {
      const d = data[index];
      const id = keyExtractor(d);
      return {
        index,
        length: heightsRef.current[id],
        offset: getHeight(data.slice(0, index))
      };
    }, [getHeight, keyExtractor])
  };
};
//# sourceMappingURL=usePrerenderedData.js.map